Stan Math Library: stan/math/prim/scal/prob/lognormal_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

lognormal_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_LOGNORMAL_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_LOGNORMAL_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/square.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <boost/random/lognormal_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19
20 namespace stan {
21 namespace math {
22
23 template <typename T_y, typename T_loc, typename T_scale>
24 typename return_type<T_y, T_loc, T_scale>::type
25 lognormal_cdf_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
26 static const char* function("stan::math::lognormal_cdf_log");
27 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
28 T_partials_return;
29
30 T_partials_return cdf_log = 0.0;
31
32 using stan::math::check_not_nan;
33 using stan::math::check_finite;
34 using stan::math::check_nonnegative;
35 using stan::math::check_positive_finite;
36 using boost::math::tools::promote_args;
37 using stan::math::value_of;
38 using std::log;
39 using std::exp;
40
41 // check if any vectors are zero length
42 if (!(stan::length(y)
43 && stan::length(mu)
44 && stan::length(sigma)))
45 return cdf_log;
46
47 check_not_nan(function, "Random variable", y);
48 check_nonnegative(function, "Random variable", y);
49 check_finite(function, "Location parameter", mu);
50 check_positive_finite(function, "Scale parameter", sigma);
51
52 OperandsAndPartials<T_y, T_loc, T_scale>
53 operands_and_partials(y, mu, sigma);
54
55 VectorView<const T_y> y_vec(y);
56 VectorView<const T_loc> mu_vec(mu);
57 VectorView<const T_scale> sigma_vec(sigma);
58 size_t N = max_size(y, mu, sigma);
59
60 const double sqrt_pi = std::sqrt(stan::math::pi());
61
62 for (size_t i = 0; i < stan::length(y); i++) {
63 if (value_of(y_vec[i]) == 0.0)
64 return operands_and_partials.value(stan::math::negative_infinity());
65 }
66
67 const double log_half = std::log(0.5);
68
69 for (size_t n = 0; n < N; n++) {
70 const T_partials_return y_dbl = value_of(y_vec[n]);
71 const T_partials_return mu_dbl = value_of(mu_vec[n]);
72 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
73 const T_partials_return scaled_diff = (log(y_dbl) - mu_dbl)
74 / (sigma_dbl * SQRT_2);
75 const T_partials_return rep_deriv = SQRT_2 / sqrt_pi
76 * exp(-scaled_diff * scaled_diff) / sigma_dbl;
77
78 // cdf_log
79 const T_partials_return erfc_calc = erfc(-scaled_diff);
80 cdf_log += log_half + log(erfc_calc);
81
82 // gradients
83 if (!is_constant_struct<T_y>::value)
84 operands_and_partials.d_x1[n] += rep_deriv / erfc_calc / y_dbl;
85 if (!is_constant_struct<T_loc>::value)
86 operands_and_partials.d_x2[n] -= rep_deriv / erfc_calc;
87 if (!is_constant_struct<T_scale>::value)
88 operands_and_partials.d_x3[n] -= rep_deriv * scaled_diff * SQRT_2
89 / erfc_calc;
90 }
91
92 return operands_and_partials.value(cdf_log);
93 }
94 }
95 }
96 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::sqrt
fvar< T > sqrt(const fvar< T > &x)
Definition: sqrt.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::lognormal_cdf_log
return_type< T_y, T_loc, T_scale >::type lognormal_cdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: lognormal_cdf_log.hpp:25
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::SQRT_2
const double SQRT_2
The value of the square root of 2, .
Definition: constants.hpp:21
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
check_finite.hpp
stan::math::erfc
fvar< T > erfc(const fvar< T > &x)
Definition: erfc.hpp:14
stan::math::pi
double pi()
Return the value of pi.
Definition: constants.hpp:86
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132

[ Stan Home Page ] © 2011–2016, Stan Development Team.